from os.path import join, normpath, dirname
import re
import sys

# nsh2py: converts nsis translations into po-files

# IMPORTANT! You must add utf-8 to the command if the src file is in utf-8.
# Usage: python nsh2po.py lang.nsh [utf-8]

# Make sure you have generated a file.pot first.
# You should always edit the po-files, when possible. If you have a
# translation in the "wrong" format, this script can be handy. Please
# continue the translation using the po-file, after converting it!
# Then use po2nsh to update/generate the nsis translation.

# Path to the directory with the translation files
translation_path = '../umenu/translations/'

# Is the source file encoded in utf-8?
source_unicode = False

# Add the path to this script in front of the translation path
translation_path = normpath(join(dirname(sys.argv[0]), translation_path))

# Some regular expressions
re_langstring = re.compile(r'LangString\W+(\w*)\W+\${LANG_\w*}\W+"(.*)"', re.I | re.S)
re_msgid = re.compile(r'msgid "(.*)"', re.I | re.S)
re_msgstr = re.compile(r'msgstr "(.*)"', re.I | re.S)
re_text = re.compile(r'"(.*)"', re.I | re.S)

def get_langcode(language):
    file = open(join(dirname(sys.argv[0]), 'languagecodes.txt'), 'rb')
    for line in file:
        if not line.startswith('#'):
            if line.find('"'+language+'"') != -1:
                length = line.find(' ')
                return line[:length]
    file.close()
    raise Exception, "Didn't find language: " + language

def get_ancicodepage(code):
    file = open(join(dirname(sys.argv[0]), 'languagecodepages.txt'), 'rb')
    for line in file:
        if not line.startswith('#'):
            if line.startswith(code+'='):
                return line[len(code)+1:]
    file.close()
    raise Exception, "Didn't find language: " + language

# Convert nsis translation into a po-file
def nsis2po(filename):
    lang = filename[:-4]  # e.g swedish
    lang_code = get_langcode(lang)  # e.g. sv
    lang_file = lang_code+'.po'  # e.g. sv.po
    print 'Converting', filename, 'to', lang_file, '...',
    
    # Parse the nsis translation we are going to convert
    translation_lang = {}
    #fp = open(filename, 'rb')
    fp = open(join(translation_path,'nsis',filename), 'rb')
    for line in fp:
        line = line.replace(r'$\r$\n', r'\n')
        m = re_langstring.search(line)
        if m != None:
            #print m.group(1), "#" ,m.group(2)
            translation_lang[m.group(1)] = m.group(2) # the langstring is the key and the text is the item here
    fp.close()
    
    # Parse the english nsis translation and construct a dictionary which
    # maps english texts to the translated texts of the same langstring
    translate = {}
    fp = open(join(translation_path,'english.nsh'), 'rb')
    for line in fp:
        line = line.replace(r'$\r$\n', r'\n')
        m = re_langstring.search(line)
        if m != None:
            if translation_lang.has_key(m.group(1)):
                translate[m.group(2)] = translation_lang[m.group(1)]
                if m.group(2) == translate[m.group(2)]:
                    translate[m.group(2)] = "" # This is no translation, it's just a copy of the english text.
            else:
                translate[m.group(2)] = ""
    fp.close()
    
    # Read the pot-file into memory
    fp = open(join(translation_path,'po','umenu.pot'), 'rb')
    lines = fp.readlines()
    fp.close()
    
    # Translate the po-file
    state = 0
    msgid = ""
    utfmsg = True
    for index in xrange(len(lines)):
        lines[index] = lines[index].replace('charset=CHARSET', 'charset=UTF-8')
        line = lines[index]
        
        if line.startswith('msgid'):
            # Find the text string from this msgid line
            m = re_msgid.search(line)
            # We are now reading an msgid
            state = 1
            msgid = m.group(1)
        elif line.startswith('"'):
            # Find the actual text string
            m = re_text.search(line)
            # Add the text to the right variable
            if state == 1:
                msgid += m.group(1)
        elif line.startswith('msgstr ""') and msgid != '':
            #print msgid, "#",translate[msgid]
            state = 0
            if msgid in translate:
                lines[index] = 'msgstr "'+ translate[msgid] +'"\n'
            else:
                lines[index] = ''
        # Convert the line to unicode
        #print lines[index]
        if source_unicode:
            lines[index] = lines[index].decode('utf-8')
        else:
		lines[index] = lines[index].decode(get_ancicodepage(lang_code))
		lines[index] = lines[index].encode('utf-8')

        #print lines[index]
    #print lines
    
    # Write text to the target file (eg. swedish.po)
    #fp = open(lang_file, "wb")
    fp = open(join(translation_path,'po',lang_file),"wb")
    fp.writelines(lines)
    fp.close()
    
    print "done"

if len(sys.argv) < 2:
    print "Usage: python nsh2po.py lang.nsh [utf-8]"
    print "Where lang.nsh is the file to be converted."
    print "You must have a file.pot first."
    print "If you add utf-8 to the command, the file will be assumed to be in utf-8."
else:
    if len(sys.argv) > 2:
        if sys.argv[2] == "utf-8": source_unicode = True
    nsis2po(sys.argv[1])
