from os.path import join, normpath, dirname
import os
import sys
import re

# po2nsh: generates nsis translations from po-files
# translator.nsh is also updated, if needed.

# Regular expressions
re_msgid = re.compile(r'msgid "(.*?)"', re.I | re.S)
re_msgstr = re.compile(r'msgstr "(.*?)"', re.I | re.S)
re_text = re.compile(r'"(.*?)"', re.I | re.S)
re_translation = re.compile(r'\${IncludeTranslation} "(\w*)"', re.I | re.S)
languagecodes = join(dirname(sys.argv[0]), 'languagecodes.txt')
languagecodepages = join(dirname(sys.argv[0]), 'languagecodepages.txt')

def get_language(code):
    file = open(languagecodes, 'rb')
    for line in file:
        if not line.startswith('#'):
            if line.startswith(code+' '):
                m = re_text.search(line)
                return m.group(1)
    file.close()
    raise Exception, "Didn't find language: %s" % code

def get_ancicodepage(code):
    file = open(languagecodepages, 'rb')
    for line in file:
        if not line.startswith('#'):
            if line.startswith(code+'='):
                return line[len(code)+1:]
    file.close()
    raise Exception, "Didn't find language: %s" % code

def po2nsh(pofile, nshdir, englishfile):
    if not os.path.exists(nshdir):
        os.mkdir(nshdir)
    if not os.path.exists(pofile):
        raise Exception("Could not find pofile %s" % pofile)
    if not os.path.exists(englishfile):
        raise Exception("Could not find englishfile %s" % englishfile)
    lang_short = os.path.basename(pofile)[:-3]
    lang = get_language(lang_short)  # e.g swedish
    nshfile = join(nshdir, lang +'.nsh')
    translator = join(os.path.dirname(englishfile),'translator.nsh')
    print 'Converting', pofile, 'to', nshfile, '...'
    
    # Read the english translation into memory
    fp = open(englishfile, 'rb')
    text = fp.read()
    fp.close()
    
    # Replace some basic info
    text = text.replace('English (default)', lang.title())
    #text = text.replace('English.nlf', lang.title()+".nlf")
    text = text.replace('${LANG_ENGLISH}', '${LANG_'+lang.upper()+'}')
    
    # Now read the po-file and translate the english language script
    fp = open(pofile, 'rb')
    state = 0
    msgstr = ""
    msgid = ""
    charset = "cp"+get_ancicodepage(lang_short);
    for line in fp:
        line = line.replace(r'\n', r'$\r$\n')
        if line.startswith('msgid'):
            # Find the text string from this msgid line
            m = re_msgid.search(line)
            # Replace the english text with the translated text
            if msgid != "" and msgstr != "":
                text = text.replace('"'+msgid+'"', '"'+msgstr+'"')
            # We are now reading an msgid
            state = 1
            msgid = m.group(1).decode('utf-8').encode(charset) # convert the text to latin-1
            msgstr = ""
        elif line.startswith('"'):
            # Find the actual text string
            m = re_text.search(line)
            # Add the text to the right variable
            if state == 1:
                try:
                    msgstr = m.group(1).decode('utf-8').encode(charset) # convert the text to latin-1            
                except UnicodeEncodeError, strerror:
                    raise Exception("Error2 encoding %s, charset=%s, pofile=%s, line=%s, err=%s" %(m.group(1),charset,pofile,line,strerror))                            
            elif state == 2:
                try:
                    msgstr = m.group(1).decode('utf-8').encode(charset) # convert the text to latin-1            
                except UnicodeEncodeError, strerror:
                    raise Exception("Error2 encoding %s, charset=%s, pofile=%s, line=%s, err=%s" %(m.group(1),charset,pofile,line,strerror))
        elif line.startswith('msgstr'):
            # Find the text string from this msgstr line
            m = re_msgstr.search(line)            
            # Add it to msgstr and start looking for more lines of msgstr text...
            try:
                msgstr = m.group(1).decode('utf-8').encode(charset) # convert the text to latin-1            
            except UnicodeEncodeError, strerror:
                raise Exception("Error2 encoding %s, charset=%s, pofile=%s, line=%s, err=%s" %(m.group(1),charset,pofile,line,strerror))
            state = 2
    # Replace the last translation too
    if msgid != "" and msgstr != "":
        text = text.replace('"'+msgid+'"', '"'+msgstr+'"')
    fp.close()
    
    # Write text to the target file (eg. swedish.nsh)
    fp = open(nshfile, 'wb')
    fp.write(text)
    fp.close()
    
    # Update translator.nsh
    fp = open(translator, 'rb')
    text = fp.read()
    fp.close()
    startpos = 0
    while True:
        # Find the next translation
        m = re_translation.search(text, startpos)
        if m == None:
            # Add translation
            if startpos != 0:
                text = text[:startpos] + '\n${IncludeTranslation} "'+lang.title()+'"' + text[startpos:]
            else:
                print "Error: failed to update translator.nsh"
            break
        startpos = m.end()
        if m.group(1) == lang.title():
            break # Found the right translation - no need to do anything!
    fp = open(translator, 'wb')
    fp.write(text)
    fp.close()
